/* This file is part of mtbe. See mtbe.c for copyright notice. */

#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "errorcode.h"
#include "lock.h"

/* Lock file name */
static char *lock_file;


/* Lock modem port using binary (bin==1) or ascii (bin==0) style.
 * path: lock file directory
 * prefix: lock file prefix; must include a leading '/' if path doesn't have
 *	   one at its end
 * device: device name to create a lock file for (without leading "/dev/")
 * Return:
 *	FATAL	malloc() failed
 *	OK	ok
 *	FAIL	already locked by someone else
 *
 * path, prefix and device are simply strcat()'ted together in this order;
 * they may be empty strings, but there must be valid pointers!
 */
int LockPort(int bin, char *path, char *prefix, char *device)
{
	int fd;			/* Lock file descriptor			*/
	FILE *fp;		/* Lock file opened for reading		*/
	pid_t pid;

	/* Create lock file name */
	lock_file = malloc(strlen(path) + strlen(prefix) + strlen(device) + 1);
	if (!lock_file)
		return FATAL;
	strcpy(lock_file, path);
	strcat(lock_file, prefix);
	strcat(lock_file, device);


	/* If file exists, read pid from it and check if the owner process
	 * still exists. If not so, remove the file.
	 */
	if ((fp = fopen(lock_file, "r")) != NULL) {
		if ((bin ? fread(&pid, sizeof(pid), 1, fp) == 1
		         : fscanf(fp, "%10d", &pid) == 1)
		    && kill(pid, 0) == -1
		    && errno == ESRCH)
			unlink(lock_file);
		fclose(fp);
	}

	/* Try to lock it */
	if ((fd = open(lock_file, O_EXCL | O_CREAT | O_RDWR, 0644)) < 0) {
		/* didn't work; clean up and return */
		free(lock_file);
		lock_file = NULL;
		return FAIL;
	}

	/* Write pid into file */
	pid = getpid();
	if (bin)
		write(fd, &pid, sizeof(pid));
	else {
		char lock_buffer[11];
		sprintf(lock_buffer, "%10d\n", pid);
		write(fd, lock_buffer, 11);
	}

	close(fd);
	return OK;
}


/* Unlock the port */
void UnlockPort(void)
{
	if (lock_file) {
		(void)unlink(lock_file);
		free(lock_file);
		lock_file = NULL;
	}
}


